

set -ex



python -c "import daal4py"
mpirun -n 4 python -m unittest discover -v -s tests -p spmd*.py
python -m unittest discover -v -s tests -p test*.py
pytest --pyargs daal4py/sklearn/
pytest --pyargs onedal/
python examples/run_examples.py
python -m daal4py examples/sycl/sklearn_sycl.py
exit 0
